package com.qmth.themis.business.service.impl;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import javax.annotation.Resource;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.themis.business.cache.RedisKeyHelper;
import com.qmth.themis.business.cache.bean.ExamRecordCacheBean;
import com.qmth.themis.business.dao.TOeExamRecordMapper;
import com.qmth.themis.business.entity.TOeExamRecord;
import com.qmth.themis.business.service.TOeExamRecordService;
import com.qmth.themis.business.util.RedisUtil;

/**
 * @Description: 考试记录 服务实现类
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
@Service
public class TOeExamRecordServiceImpl extends ServiceImpl<TOeExamRecordMapper, TOeExamRecord> implements TOeExamRecordService {

    @Resource
    TOeExamRecordMapper tOeExamRecordMapper;

    @Resource
    RedisUtil redisUtil;

    /**
     * 获取考试未完列表
     *
     * @param studentId
     * @param examId
     * @param orgId
     * @return
     */
    @Override
    public Map getUnFinishExam(Long studentId, Long examId, Long orgId) {
        Map unFinishExam = tOeExamRecordMapper.getUnFinishExam(studentId, examId, orgId);
        if (Objects.nonNull(unFinishExam)) {
            Map finalMap = new HashMap();
            Map<String, Object> waitingMap = new HashMap();
            waitingMap.put("id", unFinishExam.get("id"));
            waitingMap.put("examActivityId", unFinishExam.get("examActivityId"));
            waitingMap.put("name", unFinishExam.get("name"));
            waitingMap.put("mode", unFinishExam.get("mode"));
            waitingMap.put("preNotice", unFinishExam.get("preNotice"));
            waitingMap.put("preNoticeStaySeconds", unFinishExam.get("preNoticeStaySeconds"));
            waitingMap.put("postNotice", unFinishExam.get("postNotice"));
            finalMap.put("waiting", waitingMap);

            Map<String, Object> activityMap = new HashMap();
            activityMap.put("recordId", unFinishExam.get("recordId"));
            activityMap.put("minDurationSeconds", unFinishExam.get("minDurationSeconds"));
            activityMap.put("clientVideoPush", unFinishExam.get("clientVideoPush"));
            activityMap.put("cameraPhotoUpload", unFinishExam.get("cameraPhotoUpload"));
            activityMap.put("courseCode", unFinishExam.get("courseCode"));
            activityMap.put("wxappPhotoUpload", unFinishExam.get("wxappPhotoUpload"));
            activityMap.put("inProcessLivenessRetryCount", unFinishExam.get("inProcessLivenessRetryCount"));
            activityMap.put("examStudentId", unFinishExam.get("examStudentId"));
            activityMap.put("inProcessLivenessVerify", unFinishExam.get("inProcessLivenessVerify"));
            activityMap.put("inProcessFaceStrangerIgnore", unFinishExam.get("inProcessFaceStrangerIgnore"));
            activityMap.put("wxappVideoRecord", unFinishExam.get("wxappVideoRecord"));
            activityMap.put("inProcessFaceVerify", unFinishExam.get("inProcessFaceVerify"));
            activityMap.put("courseName", unFinishExam.get("courseName"));
            activityMap.put("wxappVideoPush", unFinishExam.get("wxappVideoPush"));
            activityMap.put("inProcessLivenessIntervalSeconds", unFinishExam.get("inProcessLivenessIntervalSeconds"));
            activityMap.put("prepareSeconds", unFinishExam.get("prepareSeconds"));
            activityMap.put("fouceFinish", unFinishExam.get("fouceFinish"));
            activityMap.put("entryAuthenticationPolicy", unFinishExam.get("entryAuthenticationPolicy"));
            activityMap.put("maxDurationSeconds", unFinishExam.get("maxDurationSeconds"));
            activityMap.put("examCount", unFinishExam.get("examCount"));
            activityMap.put("inProcessLivenessJudgePolicy", unFinishExam.get("inProcessLivenessJudgePolicy"));
            activityMap.put("id", unFinishExam.get("examActivityId"));

            finalMap.put("activity", activityMap);
            return finalMap;
        } else {
            return unFinishExam;
        }
    }

    @Transactional 
    @Override
    public Long saveByPrepare(Long examId,Long examActivityId,Long examStudentId,Long paperId,Integer serialNumber) {
    	TOeExamRecord er=new TOeExamRecord();
    	er.setExamId(examId);
    	er.setExamActivityId(examActivityId);
    	er.setExamStudentId(examStudentId);
    	er.setPaperId(paperId);
    	er.setSerialNumber(serialNumber);
    	er.setFirstPrepareTime(new Date());
    	er.setStatus(0);
    	saveOrUpdate(er);
    	ExamRecordCacheBean erCache=new ExamRecordCacheBean();
    	copy(erCache, er);
    	redisUtil.set(RedisKeyHelper.examRecordCacheKey(er.getId()), erCache);
    	return er.getId();
    }
    
    
    private void copy(ExamRecordCacheBean tb,TOeExamRecord sb) {
    	tb.setId(sb.getId());
    	tb.setExamId(sb.getExamId());
    	tb.setExamActivityId(tb.getExamActivityId());
    	tb.setPaperId(sb.getPaperId());
    	tb.setAnswerPath(sb.getAnswerPath());
    	tb.setStatus(sb.getStatus());
    	tb.setFirstPrepareTime(sb.getFirstPrepareTime());
    	tb.setFirstStartTime(sb.getFirstStartTime());
    	tb.setLastPrepareTime(sb.getLastPrepareTime());
    	tb.setLastBreakTime(sb.getLastBreakTime());
    	tb.setLastStartTime(sb.getLastStartTime());
    	tb.setLeftBreakResumeCount(sb.getLeftBreakResumeCount());
    	tb.setClientCurrentIp(sb.getClientCurrentIp());
    	tb.setClientLastSyncTime(sb.getClientLastSyncTime());
    	tb.setClientVideoPushKey(sb.getClientVideoPushKey());
    	tb.setClientVideoPushStatus(sb.getClientVideoPushStatus());
    	tb.setClientWebsocketId(sb.getClientWebsocketId());
    	tb.setClientWebsocketStatus(sb.getClientWebsocketStatus());
    	tb.setWxappLastSyncTime(sb.getWxappLastSyncTime());
    	tb.setWarningCount(sb.getWarningCount());
    	tb.setWxappVideoPushKey(sb.getWxappVideoPushKey());
    	tb.setWxappVideoPushStatus(sb.getWxappVideoPushStatus());
    	tb.setWxappWebsocketId(sb.getWxappWebsocketId());
    	tb.setWxappWebsocketStatus(sb.getWxappWebsocketStatus());
    	tb.setAnswerProgress(sb.getAnswerProgress());
    	tb.setDurationSeconds(sb.getDurationSeconds());
    	tb.setFinishTime(sb.getFinishTime());
    	tb.setFinishType(sb.getFinishType());
    	tb.setWarningCount(sb.getWarningCount());
    	tb.setReviewResult(sb.getReviewResult());
    	tb.setObjectiveScore(sb.getObjectiveScore());
    	tb.setPaperDownload(sb.getPaperDownload());
    	tb.setBreachStatus(sb.getBreachStatus());
    	tb.setPaperStruct(sb.getPaperStruct());
    	tb.setPaperStructUpload(sb.getPaperStructUpload());
    	tb.setSerialNumber(sb.getSerialNumber());
    	tb.setLastBreakId(sb.getLastBreakId());
    	tb.setEntryAuthenticationId(sb.getEntryAuthenticationId());
    	tb.setEntryAuthenticationResult(sb.getEntryAuthenticationResult());
    	tb.setInProcessFaceVerifyStatus(sb.getInProcessFaceVerifyStatus());
    	tb.setInProcessLivenessVerifyCount(sb.getInProcessLivenessVerifyCount());
    	tb.setInProcessLivenessVerifyStatus(sb.getInProcessLivenessVerifyStatus());
    }
}
